<?php

namespace Drupal\paragraphs_gridstack\Plugin\paragraphs\Behavior;

use Drupal\Core\Link;
use Drupal\paragraphs\Entity\Paragraph;
use Drupal\Core\Form\FormStateInterface;
use Drupal\paragraphs\ParagraphInterface;
use Drupal\paragraphs\ParagraphsBehaviorBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\paragraphs\Annotation\ParagraphsBehavior;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\paragraphs_gridstack\ParagraphsGridstackManagerInterface;
use Drupal\paragraphs_gridstack\GridstackBreakpointsManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Gridstack container behavior implementation.
 *
 * @ParagraphsBehavior(
 *   id = "gridstack_container",
 *   label = @Translation("Gridstack Container"),
 *   description = @Translation("Provides Gridstack layouts."),
 *   weight = 0,
 * )
 */
class GridstackContainer extends ParagraphsBehaviorBase implements ContainerFactoryPluginInterface {

  /**
   * The logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * ParagraphsGridstackManagerInterface definition.
   *
   * @var \Drupal\paragraphs_gridstack\ParagraphsGridstackManagerInterface
   */
  protected ParagraphsGridstackManagerInterface $gridstackOptionsetsManager;

  /**
   * GridstackBreakpointsManagerInterface definition.
   *
   * @var \Drupal\paragraphs_gridstack\GridstackBreakpointsManagerInterface
   */
  protected GridstackBreakpointsManagerInterface $gridstackBreakpointsManager;

  /**
   * LibraryDiscoveryInterface definition.
   *
   * @var \Drupal\Core\Asset\LibraryDiscoveryInterface
   */
  protected LibraryDiscoveryInterface $libraryDiscovery;

  /**
   * ModuleHandlerInterface definition.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * GridstackContainer plugin constructor.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityFieldManagerInterface $entity_field_manager,
    ConfigFactoryInterface $container_factory,
    ParagraphsGridstackManagerInterface $gridstack_optionsets_manager,
    GridstackBreakpointsManagerInterface $gridstack_breakpoints_manager,
    LibraryDiscoveryInterface $library_discovery,
    ModuleHandlerInterface $module_handler,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_field_manager);
    $this->configFactory = $container_factory;
    $this->gridstackOptionsetsManager = $gridstack_optionsets_manager;
    $this->gridstackBreakpointsManager = $gridstack_breakpoints_manager;
    $this->libraryDiscovery = $library_discovery;
    $this->moduleHandler = $module_handler;
    $this->logger = $logger_factory->get('paragraph_gridstack');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_field.manager'),
      $container->get('config.factory'),
      $container->get('paragraphs_gridstack.manager'),
      $container->get('paragraphs_gridstack.breakpoints_manager'),
      $container->get('library.discovery'),
      $container->get('module_handler'),
      $container->get('logger.factory'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function view(array &$build, Paragraph $paragraph, EntityViewDisplayInterface $display, $view_mode) {
    $libraries = ['paragraphs_gridstack/paragraphs_gridstack.base'];
    $settings = $paragraph->getBehaviorSetting($this->getPluginId(), 'settings', '');
    if (empty($settings)) {
      return;
    }

    $column_options = $this->buildColumnsOptions(TRUE);

    // Dynamic creation of the attached libraries.
    foreach ($settings as $breakpoint_name => $breakpoint_value) {
      foreach ($breakpoint_value as $name => $value) {
        $setting_name = substr($name, strlen($breakpoint_name) + 1);
        $attribute_name = 'data-' . preg_replace('~[\\\\/:*?"<>|._]~', '-', $name);
        switch ($setting_name) {
          case 'columns':
            // Add library with provider, received from buildColumnsOptions().
            $libraries[] = "{$column_options[$value]['provider']}/{$value}";
            $build['#attributes'][$attribute_name] = $column_options[$value]['columns'];
            break;

          case 'storage':
            $build['#attributes'][$attribute_name] = $value;
            break;
        }
      }
    }
    $libraries = array_filter(array_unique($libraries));
    $attached_libraries = $build['#attached']['library'] ?? [];
    $build['#attached']['library'] = array_merge($libraries, $attached_libraries);
  }

  /**
   * {@inheritdoc}
   */
  public function buildBehaviorForm(ParagraphInterface $paragraph, array &$form, FormStateInterface $form_state) {
    $optionsets = $this->buildOptionsetsOptions();

    $form['optionset'] = [
      '#type' => 'select',
      '#options' => $optionsets,
      '#default_value' => $paragraph->getBehaviorSetting($this->getPluginId(), 'optionset', 'paragraphs_gridstack.optionset.default'),
      '#title' => $this->t('Choose the Gridstack Optionset:'),
      '#description' => Link::createFromRoute(
        $this->t('You can manage Gridstack Optionsets here.'),
        'entity.paragraphs_gridstack.list',
      )->toRenderable(),
    ];

    $form['buttons'] = ['#type' => 'container'];
    $form['buttons']['set_by_template'] = [
      '#type' => 'button',
      '#title' => $this->t('Set by template'),
      '#value' => $this->t('Set by template'),
      '#attributes' => [
        'id' => 'pg-action-set-by-template',
        'class' => ['paragraphs-gridstack-action'],
        'title' => $this->t('Press to set elements like in template. You can manage template settings in the Gridstack Optionset.'),
      ],
    ];

    $form['buttons']['restore'] = [
      '#type' => 'button',
      '#title' => $this->t('Restore settings'),
      '#value' => $this->t('Restore settings'),
      '#attributes' => [
        'id' => 'pg-action-restore',
        'class' => ['paragraphs-gridstack-action'],
        'title' => $this->t('Press to restore settings of the latest revision.'),
      ],
    ];

    // @TODO Handle AJAX updating.
    $selected_optionset = 'default';
    /** @var \Drupal\paragraphs_gridstack\Entity\ParagraphsGridstack $optionset_entity */
    $optionset_entity = $this->gridstackOptionsetsManager->get($selected_optionset);
    $breakpoints_provider = $optionset_entity->get('breakpoints_provider');
    /** @var \Drupal\breakpoint\Breakpoint[] $breakpoints */
    $breakpoints = $this->gridstackBreakpointsManager->getBreakpointsByCondition($breakpoints_provider);
    $bp_classes = [];
    $weights = [];

    // Find a label of the max breakpoint's weight.
    foreach ($breakpoints as $breakpoint) {
      $weights[$breakpoint->pluginDefinition['label']] = $breakpoint->pluginDefinition['weight'];
    }
    $max_weight_label = array_search(max($weights), $weights);

    // Dynamic creation of the buttons.
    foreach ($breakpoints as $breakpoint) {
      $label = $breakpoint->pluginDefinition['label'];
      $bp_class = strtolower("${label}-breakpoint");
      $is_active = $label == $max_weight_label ? 'active' : '';
      $bp_classes[] = $bp_class;

      $form['buttons']['container'][]["breakpoint_${label}"] = [
        '#type' => 'button',
        '#title' => $label,
        '#value' => $label,
        '#attributes' => [
          'id' => "pg-action-${bp_class}",
          'class' => [$is_active, $bp_class, 'paragraphs-gridstack-action'],
        ],
      ];
    }

    // Get list of the available columns.
    $columns_options = $this->buildColumnsOptions();
    // Template settings of the optionset.
    $template_default = $optionset_entity->get('template');

    $mapping = [
      'columns' => [
        'type' => 'select',
        'options' => $columns_options,
        'default' => NULL,
      ],
      'template' => [
        'type' => 'textarea',
        'default' => $template_default ?? '{}',
      ],
      'previous' => [
        'type' => 'textarea',
        'default' => '{}',
      ],
      'storage' => [
        'type' => 'textarea',
        'default' => '{}',
      ],
    ];

    foreach ($breakpoints as $breakpoint) {
      // Build label of the breakpoint as plugin value.
      $form['settings'][$breakpoint->getPluginId()]["{$breakpoint->getPluginId()}.label"] = [
        '#type' => 'value',
        '#value' => $breakpoint->getLabel(),
        '#access' => FALSE,
      ];

      foreach ($mapping as $setting_key => $settings) {
        // Go through all settings and build form elements.
        $machine_name = "{$breakpoint->getPluginId()}.$setting_key";
        $default_value = $paragraph
          ->getBehaviorSetting(
            $this->getPluginId(),
            ['settings', $breakpoint->getPluginId(), $machine_name],
            $settings['default']
          );

        $label = $breakpoint->pluginDefinition['label'];
        $bp_class = strtolower("${label}-breakpoint");
        $breakpoint_settings = $label == $max_weight_label ? ['bp-settings', 'active'] : ['bp-settings'];

        $form['settings'][$breakpoint->getPluginId()][$machine_name] = [
          '#type' => $settings['type'],
          '#default_value' => $default_value,
          '#attributes' => [
            'class' => [...$breakpoint_settings, $bp_class, $machine_name],
          ],
        ];

        if (!empty($settings['options'])) {
          $form['settings'][$breakpoint->getPluginId()][$machine_name]['#options'] = $settings['options'];
        }
      }
    }

    // @TODO Check if this works.
    $form['#attached']['library'][] = 'paragraphs_gridstack/paragraphs_gridstack.base';

    $form['#attached']['drupalSettings']['paragraphs_gridstack']['breakpoint_classes'][] = array_unique($bp_classes);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(Paragraph $paragraph) {
    $settings = $paragraph->getBehaviorSetting($this->getPluginId(), 'settings', '');
    $column_options = $this->buildColumnsOptions(TRUE);

    if (empty($settings)) {
      return [];
    }

    $settings_summary = [];
    foreach ($settings as $breakpoint_name => $breakpoint_value) {
      $settings_summary[] = $this->t('<b>@breakpoint settings:</b> @settings columns', [
        '@breakpoint' => $breakpoint_value["$breakpoint_name.label"],
        '@settings' => $column_options[$breakpoint_value["$breakpoint_name.columns"]]['columns'],
      ]);
    }

    return $settings_summary;
  }

  /**
   * Return array of gridstack options sets.
   *
   * @return array
   *   Array of options.
   */
  public function buildOptionsetsOptions(): array {
    $options = [];
    $optionsets = $this->gridstackOptionsetsManager->listAll();

    foreach ($optionsets as $optionset) {
      $config = $this->configFactory->get($optionset);
      $options[$optionset] = $config->get('label');
    }

    return $options;
  }

  /**
   * Return array of available columns.
   *
   * @param bool $associate_properties
   *   Flag to return array with assoc properties.
   *
   * @return array
   *   Array of options.
   *
   * @todo think about caching of the results.
   * @see https://www.drupal.org/project/paragraphs_gridstack/issues/3324319
   */
  public function buildColumnsOptions(bool $associate_properties = FALSE): array {
    $options = [];

    $libraries_providers = ['paragraphs_gridstack'];
    $this->moduleHandler->alter('gridstack_libraries_providers', $libraries_providers);
    if (empty($libraries_providers)) {
      $this->logger->notice('No libraries providers found for Gridstack.');
    }

    foreach ($libraries_providers as $provider) {
      $librariesList = $this->libraryDiscovery
        ->getLibrariesByExtension($provider);
      // Set the provider attribute for each library.
      foreach ($librariesList as &$item) {
        $item['provider'] = $provider;
      }
      $libraries[] = $librariesList;
    }

    if (empty($libraries)) {
      $this->logger->notice('No libraries found for Gridstack.');
    }
    $libraries = array_merge(...$libraries);

    foreach ($libraries as $machine_name => $properties) {
      if (!empty($properties['gridstack'])) {
        if ($associate_properties) {
          $options[$machine_name]['label'] = $properties['label'];
          $options[$machine_name]['columns'] = $properties['columns'];
          // Set the provider value to be able to attach required library
          // for this option.
          $options[$machine_name]['provider'] = $properties['provider'];
          continue;
        }

        $options[$machine_name] = $properties['label'];
      }
    }

    return $options;
  }

}

